// $Id: Preferences.java,v 1.5 2003/03/11 22:55:58 drager Exp $

package org.daml.validator;

import java.util.*;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.BufferedWriter;
import java.io.FileWriter;

//import org.xml.sax.*;
//import org.xml.sax.helpers.*;

/**
 * Parse the preference file
 */

public class Preferences extends org.xml.sax.helpers.DefaultHandler
{

    boolean verbose = false;

    /**
     * URISubstitution structure that store the mapping from old to new URIs.
     * Used for the disk cache and for mappings & warnings about outdated URIs.
     */
    public URISubstitution outdatedURIs;

    /**
     * Keep a list of original substitution items
     */
    public HashSet subList;
    
    /**
     * Maps URIs to local cache filenames.
     */
    public HashMap cache;

    /**
     * Keep a list of original cache items
     */
    public HashSet cacheList;

    /**
     * Properties 
     */
    public Hashtable propertyList;

    /**
     * SAX Parser
     */
    String defaultParser = "org.apache.xerces.parsers.SAXParser";

    String dtdURI = "";

    /**
     * Parse the given prefernces file and return the results in a new Preference object.
     */
    public Preferences() 
    {
	outdatedURIs = new URISubstitution();
	cache = new HashMap();
	cacheList = new HashSet();
	subList = new HashSet();
	propertyList = new Hashtable();
    }

    public static Preferences parse(String file)
	throws Exception
    {
	Preferences pref = new Preferences();
	try {

	    System.out.println("Loading Validator Preferences file: "+file);

	    org.xml.sax.XMLReader parser = (org.xml.sax.XMLReader)Class.forName(pref.defaultParser).newInstance();

	    //System.out.println("Features: "+parser.getFeaturesRecognized());
	    parser.setContentHandler(pref);
	    parser.setFeature( "http://xml.org/sax/features/validation", true);
	    parser.setFeature( "http://xml.org/sax/features/namespaces", true);
	    parser.setFeature( "http://apache.org/xml/features/validation/schema", true);
	    //parser.setFeature("http://apache.org/xml/features/continue-after-fatal-error", true);

	    parser.parse(file);

	    //System.out.println("DEBUG: "+pref.outdatedURIs);
	    //System.out.println("DEBUG: "+pref.cache);
	
	    return(pref);

	} catch (org.xml.sax.SAXException e) {
	    // Make it a fatal error to have a bad preference file
	    //System.out.println("Error Reading Preference File: "+file);
	    //System.out.println(e);
	    //return(new Preferences());
	    throw e;
	}
    }

    /**
     * Add a URI and filename to the cache records.
     * This does not create the cache item.
     */
    public void addToCache(String original, String file)
    {
	cache.put(original, file);
	cache.put(normalizeURI(original), file);
	cacheList.add(original);
    }

    /**
     * Add substitution information for a URI.
     */
    public void addSubstitution(String original, String replacement, boolean warn, boolean load)
    {
	outdatedURIs.put(original, replacement, warn, load);
	outdatedURIs.put(normalizeURI(original), replacement, warn, load);
	subList.add(original);
    }

    /**
     * Set a property value
     */
    public void setProperty(String name, String value)
    {
	propertyList.put(name, value);
    }

    /**
     * Get a property's value.
     * Returns default if the property was not set.
     */
    public String getProperty(String name, String defaultVal)
    {
	String val = (String)propertyList.get(name);
	if (val == null) {
	    return defaultVal;
	} else {
	    return val;
	}
    }

    public String getProperty(String name)
    {
	return (String)propertyList.get(name);
    }

    public boolean getBooleanProperty(String name, boolean defaultVal)
    {
	String val = (String)propertyList.get(name);
	if (val == null) {
	    return defaultVal;
	} else {
	    return val.equals("true");
	}
    }

    public int getIntProperty(String name, int defaultVal)
    {
	String val = (String)propertyList.get(name);
	if (val == null) {
	    return defaultVal;
	} else {
	    return Integer.parseInt(val);
	}
    }

    /**
     * Delete a URI from the cache records.
     * This does not delete the cache file.
     */
    public void removeFromCache(String original)
    {
	cache.remove(original);
	cache.remove(normalizeURI(original));
	cacheList.remove(original);
    }


    public void startElement(java.lang.String uri, java.lang.String localName,
			     java.lang.String qName, org.xml.sax.Attributes attributes) 
	throws org.xml.sax.SAXException
    {
	
	if (localName.equals("cache")) {
	    // A cache preference contains the original uri and the local cache filename.
	    String original = attributes.getValue(dtdURI, "original");
	    String file = attributes.getValue(dtdURI, "file");
	    if (verbose) {
		System.out.println("Adding cache: "+original);
		System.out.println("              "+file);   
	    }
	    addToCache(original, file);
	} else if (localName.equals("outdatedURI")) {
	    // An outdatedURI preference contains the original URI, a replacement URI, warn flag, and load flag.
	    String original = attributes.getValue(dtdURI, "original");
	    String replacement = attributes.getValue(dtdURI, "replacement");
	    boolean warn = attributes.getValue(dtdURI, "warn").equalsIgnoreCase("true");
	    boolean load = attributes.getValue(dtdURI, "load").equalsIgnoreCase("true");
	    if (verbose) {
		System.out.println("Adding outdated: "+original);
		System.out.println("              -> "+replacement);   
	    }
	    addSubstitution(original, replacement, warn, load);
	} else if (localName.equals("property")) {
	    // A property contains a name and a value to add to the properties hashtable
	    String name = attributes.getValue(dtdURI, "name");
	    String value = attributes.getValue(dtdURI, "value");
	    if (verbose) {
		System.out.println("Setting property: "+name+" = "+value);
	    }
	    setProperty(name, value);
	} else {
	    //System.out.println("DEBUG: Unknown preference "+uri+localName);
	}
    }



    /**
     * Given a URI, convert it to a namespace string (ie, strip file extension and add ending #).
     */
    public String normalizeURI(String uri) 
    {	
	//System.out.println("URI: "+uri);
	int dotIndex = uri.lastIndexOf('.');
	int slashIndex = uri.lastIndexOf('/');
	int hashIndex = uri.lastIndexOf('#');

	if ((dotIndex == -1) || (dotIndex < slashIndex)) {
	    if (uri.endsWith("#")) {
		return uri;
	    } else if (hashIndex > slashIndex) {
		return (uri.substring(0, (1 + hashIndex)));
	    } else {
		return (uri+"#");
	    }
	} else {
	    return(uri.substring(0, dotIndex)+"#");
	}
    }

    /**
       Save the preferences file to the specified filename.
    */
    public void save(String filename) 
	throws java.io.IOException
    {
	String orig = "";

	PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
	out.println("<?xml version=\"1.0\" standalone=\"no\"?>");
	out.println("<!DOCTYPE daml-validator-preferences SYSTEM \"validator.dtd\">");
	out.println("<daml-validator-preferences>");
	out.println("");

	// outdated URIs
	// map over the subList items
	for (Iterator i = subList.iterator(); i.hasNext();) {
	    orig = (String)i.next();
	    if (verbose) { System.out.println("Substitute Item: "+orig); }
	    // Not the normalized URI so add it
	    // Q: What if the normalized URI is in the preference file? 
	    // A: It gets lost
	    // Q: Is this ok?
	    // A: Would they ever do that?
	    out.println("<outdatedURI");
	    out.println("  original=\""+orig+"\"");
	    String repl = outdatedURIs.getSubstitution(orig);
	    out.println("  replacement=\""+repl+"\"");
	    if (outdatedURIs.getWarn(orig)) {
		out.println("  warn=\"true\"");
	    } else {
		out.println("  warn=\"false\"");
	    }
	    if (outdatedURIs.getLoad(orig)) {
		out.println("  load=\"true\"");
	    } else {
		out.println("  load=\"false\"");
	    }
	    out.println("/>");
	    out.println("");

	}

	// cached URIs
	// loop over elements in cache, ignore the normalized entries.
	for (Iterator i = cacheList.iterator(); i.hasNext();) {
	    orig = (String)i.next();	    
	    if (verbose) { System.out.println("Cache Item: "+orig); }	       
	    String file=(String)cache.get(orig);
	    out.println("<cache");
	    out.println("  original=\""+orig+"\"");
	    out.println("  file=\""+file+"\"");
	    out.println("/>");
	    out.println("");
	}

	out.println("</daml-validator-preferences>");
	out.close();
    }

    /**
     * Looks in the cache for a URI and returns a cache file or the original URI.
     */
    public String cacheURI(String uri)
    {
	String result = uri;
	// Check the cache file
	if (cache.containsKey(uri)) {
	    // There exists a cache file for the URI.
	    result = (String)cache.get(uri);
	}
	return result;
    }

    /**
     * Is the uri in the cache.
     */
    public boolean hasCache(String uri)
    {	
	return cache.containsKey(uri);
    }

}
