// $Id: ValidatorAPI.java,v 1.4 2002/08/05 19:03:09 drager Exp $

package org.daml.validator;

import java.io.IOException;
import java.net.URL;
import java.util.Vector;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Enumeration;

// import org.w3c.rdf.model.Model;
// import org.w3c.rdf.model.ModelException;
// import org.w3c.rdf.implementation.syntax.sirpac.SiRPAC;
// import org.w3c.rdf.util.RDFFactory;
// import org.w3c.rdf.util.RDFUtil;
// import org.xml.sax.SAXException;
// import edu.stanford.db.rdf.model.i.RDFFactoryImpl;

import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;

import org.daml.validator.indications.Indication;
import org.daml.validator.indications.UndefinedResourceIndication;


/**
 * Implements an API to the DAML validator.
 * After adding RDF Models to validate against, any
 * RDF model can be validated.  Any number of models
 * can be validated against the backing set of models.
 */
public class ValidatorAPI {

    /**
     * Validator instance
     */
    Validator validator = null;

    /**
     * Set of RDF models to validate against
     */
    HashSet RDFModelSet = new HashSet();

    /**
     * RDF Parser
     */
//     SiRPAC parser;

    /**
     * Parser helper that loads referenced namespaces and keeps track
     * of which URIs have been loaded.
     */
    ParseHelper myParser;


    /**
     * Shows and example of how to use the ValidatorAPI.
     */
    public static void main (String args[])
	throws Exception
    {
	ValidatorAPI api = new ValidatorAPI();

	// Load models/URI to parse against
	api.addURL("http://daml.umbc.edu/ontologies/topic-ont", true);

	// Create RDF Model to validate
	String url = "http://daml.umbc.edu/ontologies/classification";

	// Validate Model
	Vector indications = api.validateURL(url);

	// Print out Indications
	for (Enumeration e = indications.elements(); e.hasMoreElements(); ) {
	    Indication i = (Indication)e.nextElement();
	    System.out.println(i.toString());
	}

    }

    

    /**
     * Generate a Validator API instance using the default preferences file.
     */
    public ValidatorAPI () 
	throws Exception
    {
	validator = new Validator();
	initialize();
    }

    /**
     * Generate a Validator API instance using a specified preferences file.
     * @param preferenceFile Pathname or URL of the preference file to use.
     */
    public ValidatorAPI (String preferenceFile) 
	throws Exception
    {
	// Create a new Validator Object
	validator = new Validator(preferenceFile);
	initialize();
    }


    /**
     * Initialize the validator.
     * This clears out all model and indication information from the validator instance.
     */
    public void initialize()
    {
	// Initialize the validator
	validator.initialize();

	// Clear the set of models to validate against
	RDFModelSet.clear();

	// Create a new parseHelper
	myParser = new ParseHelper(validator.preferences, validator.overallModel.importsPredicates, validator.datatypeValidator);
    }


    /**
     * Set the upper limit on the number of indications reported.
     * Processing stops when the limit is reached.
     * @param maxIndications The maximum number of indications to find.
     */
    public void setMaxIndications(int maxIndications) 
    {
	validator.maxIndications = maxIndications;
    }


    /**
     * Add a RDF Model to the set of models used to validate against.
     * @param model RDF Model to add to the backing model.
     */
    public void addModel(Model model) 
    {
	RDFModelSet.add(model);
    }


    /**
     * Load a URL to create a RDF Model and add that to the set of models to validate against.
     * When recurse is true then any referenced namespaces are also loaded.
     * @param url URL to load and add to the backing model.
     * @param recurse If true, referenced namespaces are also loaded.
     */
    public void addURL(URL url, boolean recurse)
	throws RDFException
    {
// 	Model m = validator.factory.createModel();
// 	parser = (SiRPAC)validator.factory.createParser();
// 	myParser.parseURI(url.toString(), parser, m, recurse);

	Model m = new ModelMem();
	myParser.parseURI(url.toString(), m, recurse);
	
	addModel(m);
    }

    /**
     * Load a URL to create a RDF Model and add that to the set of models to validate against.
     * When recurse is true then any referenced namespaces are also loaded.
     * Any XML Schema files are added to the datatype validator.
     * @param url URL to load and add to the backing model.
     * @param recurse When true, referenced namespaces are also loaded.
     */
    public void addURL(String url, boolean recurse)
	throws RDFException
    {
// 	Model m = validator.factory.createModel();
// 	parser = (SiRPAC)validator.factory.createParser();
// 	myParser.parseURI(url, parser, m, recurse);
	
	Model m = new ModelMem();
	myParser.parseURI(url, m, recurse);
	
	addModel(m);
    }


    /**
     * Validate an RDF model against the models previously added.
     * @param model Model to validate.
     * @param ident URL of the model (or some string identifier) for indication messages
     * @return A Vector of {@link Indication Indication} objects.
     */
    public Vector validateModel(Model model, String ident)
	throws RDFException
    {

	validator.reset();

	// Generate a new DAML Model to validate against
	for (Iterator i = RDFModelSet.iterator(); i.hasNext(); ) {
	    validator.addModel((Model)i.next());
	}
	
	// Validate the model
	validator.validate(model, ident);

	return validator.indications;
    }


    /**
     * Validate a daml file located at a URL against the previously
     * stored models.
     * @param url URL to load.
     * @return A Vector of {@link Indication Indication} objects.
     */
    public Vector validateURL (String url)
	throws RDFException, IOException
    {
	// Create RDF Model to validate
// 	RDFFactory f = new RDFFactoryImpl();
// 	Model m = f.createModel();
// 	SiRPAC parser = (SiRPAC)f.createParser();
// 	RDFUtil.parse(url, parser, m);

	Model m = new ModelMem();
	m.read(url);
	
	return validateModel(m, url);
    }

    
    /**
     * Validate a daml file located at a URL against the previously
     * stored models.
     * @param url URL to load.
     * @return A Vector of {@link Indication Indication} objects.
     */
    public Vector validateURL (URL url)
	throws RDFException, IOException
    {
	return validateURL(url.toString());
    }

}



    





